# -*- coding: utf-8 -*-

from six.moves.urllib.parse import urlparse, urlunparse, urljoin, urlencode

from wargaming import settings
from wargaming.exceptions import ValidationError


class MetaAPI(type):

    def __new__(cls, name, bases, attrs):
        attrs['_subclasses'] = {}
        new_cls = super(MetaAPI, cls).__new__(cls, name, bases, attrs)

        for name, sub_api in attrs.items():
            if not isinstance(sub_api, SubclassAPI):
                continue

            new_cls._subclasses[name] = sub_api

        return new_cls


class BaseAPI(object):
    _subclasses = None

    def __init__(self, application_id, base_url):
        self.application_id = application_id

        url_parts = urlparse(base_url)
        self.url_scheme = url_parts.scheme
        self.url_netloc = url_parts.netloc
        self.url_path = url_parts.path

        for subclass in self._subclasses.values():
            subclass.api = self

    def _get_url(self, path, parameters):
        return urlunparse([
            self.url_scheme,
            self.url_netloc,
            urljoin(self.url_path, path),
            None,
            urlencode(parameters),
            None
        ])

    @staticmethod
    def prepare_language(value):
        """Localization language

        One of the :data:`wargaming.settings.ALLOWED_LANGUAGES`
        """

        if value not in settings.ALLOWED_LANGUAGES:
            raise ValidationError('Invalid language: {}'.format(value))

        return value

    @staticmethod
    def prepare_fields(value):
        """List of response fields

        Fields are separated by commas. Nested fields are separated by dots."""

        if value:
            return ','.join(value)

        return ''


class SubclassAPI(object):
    """Base class for all of the API endpoint 'namespaces'"""

    def __init__(self):
        self.api = None



# -*- coding: utf-8 -*-


class APIError(Exception):
    pass


class RequestError(APIError):

    def __init__(self, code, field, message, value):
        self.status_code = code
        self.field = field
        self.message = message
        self.value = value

    def __str__(self):
        return self.value

    def __repr__(self):
        return '<Request error: {}>'.format(self.message)


class ValidationError(APIError):
    pass



# -*- coding: utf-8 -*-

import requests
from six.moves.urllib.parse import urlparse, urlunparse, urlencode

from wargaming import settings
from wargaming.exceptions import RequestError
from wargaming.api import BaseAPI, SubclassAPI

__all__ = ('bind',)


def __init__(self, parent):
    if isinstance(parent, SubclassAPI):
        self.api = parent.api
    else:
        self.api = parent


def execute(self, **kwargs):
    # Build GET parameters for query
    parameters = {}

    for key, value in kwargs.items():
        if value is None:
            continue

        prepare_func = getattr(self.api, 'prepare_{}'.format(key), None)
        if prepare_func is not None:
            value = prepare_func(value)

        parameters[key] = value

    parameters.update({
        'application_id': self.api.application_id,
    })

    url = self.api._get_url(self.path, parameters)

    response = requests.get(url, headers={
        'User-Agent': settings.HTTP_USER_AGENT_HEADER,
    }).json()

    if response['status'] == 'error':
        raise RequestError(**response['error'])

    return response['data']


def bind(**config):
    docstring = config.pop('doc', None)

    properties = {
        'path': config['path'],
        'allowed_params': config['allowed_params'],
        '__init__': __init__,
        'execute': execute,
    }

    cls = type('API{}Method'.format(config['path'].title().replace('/', '')),
               (object, ), properties)

    def _call(self, *args, **kwargs):
        return cls(self).execute(*args, **kwargs)

    _call.__name__ = config['path'].strip('/').replace('/', '_').lower()
    if docstring is not None:
        _call.__doc__ = docstring

    return _call



# -*- coding: utf-8 -*-

import six

from wargaming.api import BaseAPI, MetaAPI
from wargaming.games.wot.accounts import Accounts
from wargaming.games.wot.auth import Auth


__all__ = ('API', )


@six.add_metaclass(MetaAPI)
class API(BaseAPI):

    def __init__(self, application_id, base_url='https://api.worldoftanks.com/wot/'):
        super(API, self).__init__(application_id, base_url)

    accounts = Accounts()

    auth = Auth()



# -*- coding: utf-8 -*-

import unittest2 as unittest

from wargaming.games.wot import API
from wargaming.exceptions import ValidationError


class APITestCase(unittest.TestCase):

    def setUp(self):
        self.api = API(application_id='demo')

    def test_invalid_language(self):
        self.assertRaises(ValidationError, self.api.accounts.list,
                          search='test', language='tlh')  # Klingon lang ;)



# -*- coding: utf-8 -*-

import unittest2 as unittest

from wargaming.games.wot import API


class WOTTestCase(unittest.TestCase):

    def setUp(self):
        self.api = API(application_id='demo')

    def test_list_of_players(self):
        response = self.api.accounts.list(search='demo', limit=5)
        self.assertEqual(5, len(response))
